@@ -6,6 +6,8 @@
 /* SYNOPSIS
 /*	#include <cleanup.h>
 /*
+/*	void	cleanup_milter_header_checks_init(void)
+/*
 /*	void	cleanup_milter_receive(state, count)
 /*	CLEANUP_STATE *state;
 /*	int	count;
@@ -32,6 +34,9 @@
 /*	filter (milter) applications, including in-place queue file
 /*	modification.
 /*
+/*	cleanup_milter_header_checks_init() does pre-jail
+/*	initializations.
+/*
 /*	cleanup_milter_receive() receives mail filter definitions,
 /*	typically from an smtpd(8) server process, and registers
 /*	local call-back functions for macro expansion and for queue
@@ -222,6 +227,8 @@
 
 /*#define msg_verbose	2*/
 
+static HBC_CHECKS *cleanup_milter_hbc_checks;
+static VSTRING *cleanup_milter_hbc_reply;
 static void cleanup_milter_set_error(CLEANUP_STATE *, int);
 static const char *cleanup_add_rcpt_par(void *, const char *, const char *);
 
@@ -345,9 +352,9 @@ static char *cleanup_milter_hbc_extend(void *context, const char *command,
 	    state->errs |= CLEANUP_STAT_CONT;
 	state->flags &= ~CLEANUP_FLAG_FILTER_ALL;
 	cleanup_milter_hbc_log(context, "reject", where, buf, state->reason);
-	vstring_sprintf(state->milter_hbc_reply, "%d %s",
+	vstring_sprintf(cleanup_milter_hbc_reply, "%d %s",
 			detail->smtp, state->reason);
-	STR(state->milter_hbc_reply)[0] = *state->reason;
+	STR(cleanup_milter_hbc_reply)[0] = *state->reason;
 	return ((char *) buf);
     }
     if (STREQUAL(command, "FILTER", cmd_len)) {
@@ -368,7 +375,7 @@ static char *cleanup_milter_hbc_extend(void *context, const char *command,
     }
     if (STREQUAL(command, "DISCARD", cmd_len)) {
 	cleanup_milter_hbc_log(context, "discard", where, buf, optional_text);
-	vstring_strcpy(state->milter_hbc_reply, "D");
+	vstring_strcpy(cleanup_milter_hbc_reply, "D");
 	state->flags |= CLEANUP_FLAG_DISCARD;
 	state->flags &= ~CLEANUP_FLAG_FILTER_ALL;
 	return ((char *) buf);
@@ -409,7 +416,7 @@ static int cleanup_milter_header_checks(CLEANUP_STATE *state, VSTRING *buf)
      * end-of-message stage, therefore all the header operations are relative
      * to the primary message header.
      */
-    ret = hbc_header_checks((void *) state, state->milter_hbc_checks,
+    ret = hbc_header_checks((void *) state, cleanup_milter_hbc_checks,
 			    MIME_HDR_PRIMARY, (HEADER_OPTS *) 0,
 			    buf, (off_t) 0);
     if (ret == 0) {
@@ -504,10 +511,14 @@ static void cleanup_milter_hbc_add_meta_records(CLEANUP_STATE *state)
 
 /* cleanup_milter_header_checks_init - initialize post-Milter header checks */
 
-static void cleanup_milter_header_checks_init(CLEANUP_STATE *state)
+void    cleanup_milter_header_checks_init(void)
 {
-#define NO_NESTED_HDR_MAPS	((MAPS *) 0)
-#define NO_MIME_HDR_MAPS	((MAPS *) 0)
+    static const char myname[] = "cleanup_milter_header_checks_init";
+
+#define NO_NESTED_HDR_NAME	""
+#define NO_NESTED_HDR_VALUE	""
+#define NO_MIME_HDR_NAME	""
+#define NO_MIME_HDR_VALUE	""
 
     static /* XXX not const */ HBC_CALL_BACKS call_backs = {
 	cleanup_milter_hbc_log,
@@ -515,12 +526,47 @@ static void cleanup_milter_header_checks_init(CLEANUP_STATE *state)
 	cleanup_milter_hbc_extend,
     };
 
-    state->milter_hbc_checks =
-	hbc_header_checks_create(cleanup_milt_head_checks,
-				 NO_MIME_HDR_MAPS,
-				 NO_NESTED_HDR_MAPS,
+    if (*var_milt_head_checks == 0)
+	msg_panic("%s: %s is empty", myname, VAR_MILT_HEAD_CHECKS);
+
+    if (cleanup_milter_hbc_checks)
+	msg_panic("%s: cleanup_milter_hbc_checks is not null", myname);
+    cleanup_milter_hbc_checks =
+	hbc_header_checks_create(VAR_MILT_HEAD_CHECKS, var_milt_head_checks,
+				 NO_MIME_HDR_NAME, NO_MIME_HDR_VALUE,
+				 NO_NESTED_HDR_NAME, NO_NESTED_HDR_VALUE,
 				 &call_backs);
-    state->milter_hbc_reply = vstring_alloc(100);
+
+    if (cleanup_milter_hbc_reply)
+	msg_panic("%s: cleanup_milter_hbc_reply is not null", myname);
+    cleanup_milter_hbc_reply = vstring_alloc(100);
+}
+
+#ifdef TEST
+
+/* cleanup_milter_header_checks_deinit - undo cleanup_milter_header_checks_init */
+
+static void cleanup_milter_header_checks_deinit(void)
+{
+    static const char myname[] = "cleanup_milter_header_checks_deinit";
+
+    if (cleanup_milter_hbc_checks == 0)
+	msg_panic("%s: cleanup_milter_hbc_checks is null", myname);
+    hbc_header_checks_free(cleanup_milter_hbc_checks);
+    cleanup_milter_hbc_checks = 0;
+
+    if (cleanup_milter_hbc_reply == 0)
+	msg_panic("%s: cleanup_milter_hbc_reply is null", myname);
+    vstring_free(cleanup_milter_hbc_reply);
+    cleanup_milter_hbc_reply = 0;
+}
+
+#endif
+
+/* cleanup_milter_header_checks_reinit - re-init post-Milter header checks */
+
+static void cleanup_milter_header_checks_reinit(CLEANUP_STATE *state)
+{
     if (state->filter)
 	myfree(state->filter);
     state->filter = 0;
@@ -533,12 +579,6 @@ static void cleanup_milter_header_checks_init(CLEANUP_STATE *state)
 
 static void cleanup_milter_hbc_finish(CLEANUP_STATE *state)
 {
-    if (state->milter_hbc_checks)
-	hbc_header_checks_free(state->milter_hbc_checks);
-    state->milter_hbc_checks = 0;
-    if (state->milter_hbc_reply)
-	vstring_free(state->milter_hbc_reply);
-    state->milter_hbc_reply = 0;
     if (CLEANUP_OUT_OK(state)
 	&& !CLEANUP_MILTER_REJECTING_OR_DISCARDING_MESSAGE(state)
 	&& (state->filter || state->redirect))
@@ -642,7 +682,7 @@ static const char *cleanup_add_header(void *context, const char *name,
      */
     buf = vstring_alloc(100);
     vstring_sprintf(buf, "%s:%s%s", name, space, value);
-    if (state->milter_hbc_checks) {
+    if (cleanup_milter_hbc_checks) {
 	if (cleanup_milter_header_checks(state, buf) == 0
 	    || (state->flags & CLEANUP_FLAG_DISCARD)) {
 	    vstring_free(buf);
@@ -705,8 +745,8 @@ static const char *cleanup_add_header(void *context, const char *name,
      * In case of error while doing record output.
      */
     return (CLEANUP_OUT_OK(state) == 0 ? cleanup_milter_error(state, 0) :
-	    state->milter_hbc_reply && LEN(state->milter_hbc_reply) ?
-	    STR(state->milter_hbc_reply) : 0);
+	    cleanup_milter_hbc_reply && LEN(cleanup_milter_hbc_reply) ?
+	    STR(cleanup_milter_hbc_reply) : 0);
 
     /*
      * Note: state->append_hdr_pt_target never changes.
@@ -1030,7 +1070,7 @@ static const char *cleanup_patch_header(CLEANUP_STATE *state,
      * be dropped.
      */
     vstring_sprintf(buf, "%s:%s%s", new_hdr_name, hdr_space, new_hdr_value);
-    if (state->milter_hbc_checks
+    if (cleanup_milter_hbc_checks
 	&& cleanup_milter_header_checks(state, buf) == 0)
 	CLEANUP_PATCH_HEADER_RETURN(0);
 
@@ -1099,8 +1139,8 @@ static const char *cleanup_patch_header(CLEANUP_STATE *state,
      */
     CLEANUP_PATCH_HEADER_RETURN(
 	       CLEANUP_OUT_OK(state) == 0 ? cleanup_milter_error(state, 0) :
-		   state->milter_hbc_reply && LEN(state->milter_hbc_reply) ?
-				STR(state->milter_hbc_reply) : 0);
+		 cleanup_milter_hbc_reply && LEN(cleanup_milter_hbc_reply) ?
+				STR(cleanup_milter_hbc_reply) : 0);
 
     /*
      * Note: state->append_hdr_pt_target never changes.
@@ -1982,8 +2022,8 @@ static const char *cleanup_milter_apply(CLEANUP_STATE *state, const char *event,
      * Don't process our own milter_header/body checks replies. See comments
      * in cleanup_milter_hbc_extend().
      */
-    if (state->milter_hbc_reply &&
-	strcmp(resp, STR(state->milter_hbc_reply)) == 0)
+    if (cleanup_milter_hbc_reply &&
+	strcmp(resp, STR(cleanup_milter_hbc_reply)) == 0)
 	return (0);
 
     /*
@@ -2148,8 +2188,8 @@ void    cleanup_milter_inspect(CLEANUP_STATE *state, MILTERS *milters)
     /*
      * Prologue: prepare for Milter header/body checks.
      */
-    if (cleanup_milt_head_checks)
-	cleanup_milter_header_checks_init(state);
+    if (*var_milt_head_checks)
+	cleanup_milter_header_checks_reinit(state);
 
     /*
      * Process mail filter replies. The reply format is verified by the mail
@@ -2162,7 +2202,7 @@ void    cleanup_milter_inspect(CLEANUP_STATE *state, MILTERS *milters)
     /*
      * Epilogue: finalize Milter header/body checks.
      */
-    if (cleanup_milt_head_checks)
+    if (*var_milt_head_checks)
 	cleanup_milter_hbc_finish(state);
 
     if (msg_verbose)
@@ -2319,7 +2359,6 @@ char   *var_milt_daemon_name = "host.example.com";
 char   *var_milt_v = DEF_MILT_V;
 MILTERS *cleanup_milters = (MILTERS *) ((char *) sizeof(*cleanup_milters));
 char   *var_milt_head_checks = "";
-MAPS   *cleanup_milt_head_checks;
 
 /* Dummies to satisfy unused external references. */
 
@@ -2586,11 +2625,10 @@ int     main(int unused_argc, char **argv)
 		cleanup_milter_hbc_finish(state);
 		myfree(var_milt_head_checks);
 		var_milt_head_checks = "";
-		maps_free(cleanup_milt_head_checks);
-		cleanup_milt_head_checks = 0;
+		cleanup_milter_header_checks_deinit();
 	    }
 	    close_queue_file(state);
-	} else if (state->milter_hbc_reply && LEN(state->milter_hbc_reply)) {
+	} else if (cleanup_milter_hbc_reply && LEN(cleanup_milter_hbc_reply)) {
 	    /* Postfix libmilter would skip further requests. */
 	    msg_info("ignoring: %s %s %s", argv->argv[0],
 		     argv->argc > 1 ? argv->argv[1] : "",
@@ -2695,9 +2733,7 @@ int     main(int unused_argc, char **argv)
 		msg_warn("can't change header checks");
 	    } else {
 		var_milt_head_checks = mystrdup(argv->argv[1]);
-		cleanup_milt_head_checks = maps_create(VAR_MILT_HEAD_CHECKS,
-				      var_milt_head_checks, DICT_FLAG_LOCK);
-		cleanup_milter_header_checks_init(state);
+		cleanup_milter_header_checks_init();
 	    }
 	} else if (strcmp(argv->argv[0], "sender_bcc_maps") == 0) {
 	    if (argv->argc != 2) {
@@ -2729,10 +2765,8 @@ int     main(int unused_argc, char **argv)
 	    msg_info("errs = %s", cleanup_strerror(state->errs));
     }
     cleanup_state_free(state);
-    if (*var_milt_head_checks) 
+    if (*var_milt_head_checks)
 	myfree(var_milt_head_checks);
-    if (cleanup_milt_head_checks)
-	maps_free(cleanup_milt_head_checks);
 
     return (0);
 }
